import { world, BlockPermutation } from '@minecraft/server';

//DOORS

//Black Door
const processedBlocks = new Map();

world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_black_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:black_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:black_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:black_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:black_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:black_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:black_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:black_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:black_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:black_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:black_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:black_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});



//Brown Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_brown_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:brown_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:brown_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:brown_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:brown_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:brown_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:brown_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:brown_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:brown_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:brown_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:brown_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:brown_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Blue Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_blue_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:blue_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:blue_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:blue_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:blue_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:blue_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:blue_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:blue_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:blue_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:blue_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:blue_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:blue_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Cyan Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_cyan_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:cyan_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:cyan_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:cyan_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:cyan_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:cyan_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:cyan_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:cyan_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:cyan_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:cyan_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:cyan_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:cyan_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Dark Green Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_dark_green_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:dark_green_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:dark_green_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:dark_green_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:dark_green_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:dark_green_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:dark_green_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:dark_green_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:dark_green_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:dark_green_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:dark_green_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:dark_green_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Dark Purple Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_dark_purple_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:dark_purple_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:dark_purple_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:dark_purple_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:dark_purple_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:dark_purple_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:dark_purple_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:dark_purple_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:dark_purple_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:dark_purple_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:dark_purple_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:dark_purple_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Green Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_green_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:green_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:green_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:green_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:green_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:green_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:green_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:green_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:green_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:green_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:green_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:green_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Grey Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_grey_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:grey_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:grey_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:grey_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:grey_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:grey_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:grey_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:grey_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:grey_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:grey_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:grey_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:grey_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Light Blue Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_light_blue_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:light_blue_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:light_blue_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:light_blue_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:light_blue_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:light_blue_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:light_blue_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:light_blue_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:light_blue_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:light_blue_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:light_blue_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:light_blue_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Light Grey Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_light_grey_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:light_grey_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:light_grey_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:light_grey_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:light_grey_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:light_grey_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:light_grey_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:light_grey_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:light_grey_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:light_grey_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:light_grey_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:light_grey_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Orange Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_orange_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:orange_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:orange_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:orange_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:orange_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:orange_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:orange_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:orange_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:orange_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:orange_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:orange_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:orange_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Pink Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_pink_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:pink_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:pink_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:pink_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:pink_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:pink_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:pink_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:pink_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:pink_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:pink_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:pink_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:pink_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Purple Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_purple_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:purple_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:purple_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:purple_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:purple_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:purple_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:purple_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:purple_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:purple_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:purple_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:purple_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:purple_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Red Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_red_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:red_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:red_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:red_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:red_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:red_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:red_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:red_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:red_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:red_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:red_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:red_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//White Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_white_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:white_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:white_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:white_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:white_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:white_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:white_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:white_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:white_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:white_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:white_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:white_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});

//Yellow Door
world.beforeEvents.worldInitialize.subscribe(eventData => {
    eventData.blockComponentRegistry.registerCustomComponent('old_villagers_plus:on_place_yellow_door', {
        onPlace(e) {
            const { block } = e;
            const aboveBlock = block.above();
            const northBlock = block.north(); // Get the block to the east
            const southBlock = block.south(); // Get the block to the east
            const eastBlock = block.east(); // Get the block to the east
            const westBlock = block.west(); // Get the block to the east
            const { x, y, z } = block.location;
            if (processedBlocks.has(`${x},${y},${z}`)) {
                return; // Exit early if block has already been processed
            }
            const currentStates = block.permutation.getAllStates();
            const cardinalDirection = currentStates['minecraft:cardinal_direction'];
            if (aboveBlock.typeId === 'minecraft:air' && !block.permutation.getState('old_villagers_plus:upper_block_bit')) {
                const aboveBlockPermutation = BlockPermutation.resolve('old_villagers_plus:yellow_door', {
                    'old_villagers_plus:upper_block_bit': true,
                    'minecraft:cardinal_direction': cardinalDirection
                });
                aboveBlock.setPermutation(aboveBlockPermutation);
                const newPermutation = BlockPermutation.resolve(block.typeId, currentStates);
                block.setPermutation(newPermutation);
            }
            if (cardinalDirection === 'south' && eastBlock?.typeId === 'old_villagers_plus:yellow_door') {
                const eastBlockStates = eastBlock.permutation.getAllStates();
                if (!eastBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const eastBlockAbove = aboveBlock.east();
                if (
                    aboveBlockCardinalDirection === 'south' &&
                    eastBlockAbove?.typeId === 'old_villagers_plus:yellow_door'
                ) {
                    const eastBlockAboveStates = eastBlockAbove.permutation.getAllStates();
                    if (!eastBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...eastBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'east' && northBlock?.typeId === 'old_villagers_plus:yellow_door') {
                const northBlockStates = northBlock.permutation.getAllStates();
                if (!northBlockStates['old_villagers_plus:door_hinge_bit']) {
                    // Change the alpha_livestock:door_hinge_bit state of the placed block to true
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const northBlockAbove = aboveBlock.north();
                if (
                    aboveBlockCardinalDirection === 'east' &&
                    northBlockAbove?.typeId === 'old_villagers_plus:yellow_door'
                ) {
                    const northBlockAboveStates = northBlockAbove.permutation.getAllStates();
                    if (!northBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...northBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'north' && westBlock?.typeId === 'old_villagers_plus:yellow_door') {
                const westBlockStates = westBlock.permutation.getAllStates();
                if (!westBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const westBlockAbove = aboveBlock.west();
                if (
                    aboveBlockCardinalDirection === 'north' &&
                    westBlockAbove?.typeId === 'old_villagers_plus:yellow_door'
                ) {
                    const westBlockAboveStates = westBlockAbove.permutation.getAllStates();
                    if (!westBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...westBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            if (cardinalDirection === 'west' && southBlock?.typeId === 'old_villagers_plus:yellow_door') {
                const southBlockStates = southBlock.permutation.getAllStates();
                if (!southBlockStates['old_villagers_plus:door_hinge_bit']) {
                    const newPermutation = BlockPermutation.resolve(block.typeId, {
                        ...currentStates,
                        'old_villagers_plus:door_hinge_bit': true
                    });
                    block.setPermutation(newPermutation);
                }
                const aboveBlockCardinalDirection = aboveBlock.permutation.getState('minecraft:cardinal_direction');
                const southBlockAbove = aboveBlock.south();
                if (
                    aboveBlockCardinalDirection === 'west' &&
                    southBlockAbove?.typeId === 'old_villagers_plus:yellow_door'
                ) {
                    const southBlockAboveStates = southBlockAbove.permutation.getAllStates();
                    if (!southBlockAboveStates['old_villagers_plus:door_hinge_bit']) {
                        const newAbovePermutation = BlockPermutation.resolve(aboveBlock.typeId, {
                            ...southBlockAboveStates,
                            'old_villagers_plus:door_hinge_bit': true
                        });
                        aboveBlock.setPermutation(newAbovePermutation);
                    }
                }
            }
            processedBlocks.set(`${x},${y},${z}`, true);
        }
    });
});

// Subscribe to the 'playerBreakBlock' event to remove processed block entry
world.afterEvents.playerBreakBlock.subscribe(eventData => {
    const { block } = eventData;
    const { x, y, z } = block.location;

    // Remove the block's entry from the processedBlocks map
    processedBlocks.delete(`${x},${y},${z}`);

    // Check if the block above or below the destroyed block is also alpha_livestock:door
    const aboveBlock = block.above();
    const belowBlock = block.below();
    if (belowBlock.typeId === 'old_villagers_plus:yellow_door') {
        belowBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y - 1},${z}`); // Remove the below block's entry
    } else if (aboveBlock.typeId === 'old_villagers_plus:yellow_door') {
        aboveBlock.setType('minecraft:air');
        processedBlocks.delete(`${x},${y + 1},${z}`); // Remove the above block's entry
    }
});